package com.sun.media.sound;

import javax.sound.midi.ShortMessage;
import javax.sound.midi.MidiMessage;
import javax.sound.midi.Receiver;
import javax.sound.midi.MidiUnavailableException;

final class MidiOutDevice extends AbstractMidiDevice
{
    MidiOutDevice(AbstractMidiDeviceProvider.Info info) {
        super(info);
    }
    
    @Override
    protected synchronized void implOpen() throws MidiUnavailableException {
        this.id = this.nOpen(((AbstractMidiDeviceProvider.Info)this.getDeviceInfo()).getIndex());
        if (this.id == 0L) {
            throw new MidiUnavailableException("Unable to open native device");
        }
        return;
    }
    
    @Override
    protected synchronized void implClose() {
        long id = this.id;
        this.id = 0L;
        super.implClose();
        this.nClose(id);
    }
    
    @Override
    public long getMicrosecondPosition() {
        long nGetTimeStamp = -1L;
        if (this.isOpen()) {
            nGetTimeStamp = this.nGetTimeStamp(this.id);
        }
        return nGetTimeStamp;
    }
    
    @Override
    protected boolean hasReceivers() {
        return true;
    }
    
    @Override
    protected Receiver createReceiver() {
        return new MidiOutDevice.MidiOutReceiver();
    }
    
    private native long nOpen(int p0) throws MidiUnavailableException;
    
    private native void nClose(long p0);
    
    private native void nSendShortMessage(long p0, int p1, long p2);
    
    private native void nSendLongMessage(long p0, byte[] p1, int p2, long p3);
    
    private native long nGetTimeStamp(long p0);
    
    static void access$000(MidiOutDevice midiOutDevice, long n, int n2, long n3) {
        midiOutDevice.nSendShortMessage(n, n2, n3);
    }
    
    static void access$100(MidiOutDevice midiOutDevice, long n, byte[] array, int n2, long n3) {
        midiOutDevice.nSendLongMessage(n, array, n2, n3);
    }
    
    final class MidiOutReceiver extends AbstractMidiDevice.AbstractReceiver
    {
        @Override
        void implSend(MidiMessage midiMessage, long n) {
            int length = midiMessage.getLength();
            int status = midiMessage.getStatus();
            if (length <= 3 && status != 240 && status != 247) {
                int packedMsg;
                if ((midiMessage instanceof ShortMessage)) {
                    if ((midiMessage instanceof FastShortMessage)) {
                        packedMsg = ((FastShortMessage)midiMessage).getPackedMsg();
                    }
                    else {
                        ShortMessage shortMessage = (ShortMessage)midiMessage;
                        packedMsg = ((status & 0xFF) | (shortMessage.getData1() & 0xFF) << 8 | (shortMessage.getData2() & 0xFF) << 16);
                    }
                }
                else {
                    packedMsg = 0;
                    byte[] message = midiMessage.getMessage();
                    if (length > 0) {
                        packedMsg = (message[0] & 0xFF);
                        if (length > 1) {
                            if (status == 255) {
                                return;
                            }
                            packedMsg |= (message[1] & 0xFF) << 8;
                            if (length > 2) {
                                packedMsg |= (message[2] & 0xFF) << 16;
                            }
                        }
                    }
                }
                MidiOutDevice.access$000(MidiOutDevice.this, MidiOutDevice.this.id, packedMsg, n);
            }
            else {
                byte[] array;
                if ((midiMessage instanceof FastSysexMessage)) {
                    array = ((FastSysexMessage)midiMessage).getReadOnlyMessage();
                }
                else {
                    array = midiMessage.getMessage();
                }
                int min = Math.min(length, array.length);
                if (min > 0) {
                    MidiOutDevice.access$100(MidiOutDevice.this, MidiOutDevice.this.id, array, min, n);
                }
            }
        }
        
        synchronized void sendPackedMidiMessage(int n, long n2) {
            if (this.isOpen() && MidiOutDevice.this.id != 0L) {
                MidiOutDevice.access$000(MidiOutDevice.this, MidiOutDevice.this.id, n, n2);
            }
        }
    }
}
